package com.blazinggames.retro.modern.search;

// Java imports
import java.awt.*;

// Spelchan imports
import com.blazinggames.urcl.ui.sprite.*;

/**
 * used by the Hide and Seek game to draw the arrows.
 *
 * @author Billy D. Spelchan
 * @version 1.00.010914
*/
/**
 * Ultimate Retro Episode 3: Classic Hide and Seek
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * Draws an arrow of any size facing one of eoght directions.
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class ArrowSprite 
extends Sprite
{
// *********************
// ***** CONSTANTS *****
// *********************
    public static final int STATE_LOCATION = 0;
    public static final int STATE_NORTH = 1;
    public static final int STATE_NORTHEAST = 2;
    public static final int STATE_EAST = 3;
    public static final int STATE_SOUTHEAST = 4;
    public static final int STATE_SOUTH = 5;
    public static final int STATE_SOUTHWEST = 6;
    public static final int STATE_WEST = 7;
    public static final int STATE_NORTHWEST = 8;
    
// *********************
// ***** VARIABLES *****
// *********************

    int _state;
    Color _color;

    int _xpoints[] = new int[4];
    int _ypoints[] = new int[4];
    
    Point _gameLocation;
    
// ************************
// ***** CONSTRUCTORs *****
// ************************
    
    ArrowSprite(Color c, int state, int x, int y, int w, int h)
    {
        super();
        _state = state;
        _color = c;
        setBounds(x, y, w, h);
        _gameLocation = new Point(0,0);
    }


// ****************************************************************
// ***** METHODS - additional Information setting and getting *****
// ****************************************************************

    public void setState(int n)
    {
        _state = n;
        invalidate();
    }
    
    public int getState()
    {
        return _state;
    }
    
    public void setColor(Color c)
    {
        _color = c;
        invalidate();
    }

    public void setGameLocation(int x, int y)
    {
        _gameLocation.setLocation(x, y);
    }
    
    public Point getGameLocation()
    {
        return _gameLocation;
    }

// *********************************
// ***** METHODS - From Sprite *****
// *********************************

    /**
     * Draws the sprite on the provided graphics context. It should be noted that sprites ARE NOT required
     * to store the state of any graphics settings so it is possible that the graphics settings will
     * change. Sprite managers should store any settings for the graphics context before rendering the
     * sprites and restore them afterwards. Overriders may want to call this to automatically handle
     * drawing the background and bounding box
     *
     * @param g the graphics context to draw to
     * @param c component required by some awt draw methods
    */
    public void render(Graphics g, Component c)
    {
        render(g, c, Sprite.NO_OFFSET);
    }
    
    /**
     * Same as render, but adjusts the sprites coordinates by adding offset to coordinates.
     *
     * @param g the graphics context to draw to
     * @param c component required by some awt draw methods
     * @param offset point relative to top left that sprite is to be drawn
    */
    public void render(Graphics g, Component c, Point offset)
    {
        super.render(g, c, offset);
        if (_isVisible)
        {
            g.setColor(_color);
            calculatePoints(_bounds.x + offset.x, _bounds.y + offset.y,
                        _bounds.x + offset.x + _bounds.width, _bounds.y + offset.y + _bounds.height);
            g.fillPolygon(_xpoints, _ypoints, 4);
        }
    }


    public void calculatePoints(int x1, int y1, int x2, int y2)
    {
        int tempx, tempy;
        
        switch (_state)
        {
            case STATE_LOCATION:
                _xpoints[0] = x1;
                _ypoints[0] = y1;
                _xpoints[1] = x2;
                _ypoints[1] = y1;
                _xpoints[2] = x2;
                _ypoints[2] = y2;
                _xpoints[3] = x1;
                _ypoints[3] = y2;
                break;
                
            case STATE_NORTH:
                tempx = (x1 + x2 + 1) / 2;
                tempy = (y1 + y2  + 1) / 2;
                _xpoints[0] = tempx;
                _ypoints[0] = y1;
                _xpoints[1] = x2;
                _ypoints[1] = tempy;
                _xpoints[2] = x1;
                _ypoints[2] = tempy;
                _xpoints[3] = tempx;
                _ypoints[3] = y1;
                break;

            case STATE_NORTHEAST:
                _xpoints[0] = x1;
                _ypoints[0] = y1;
                _xpoints[1] = x2;
                _ypoints[1] = y1;
                _xpoints[2] = x2;
                _ypoints[2] = y2;
                _xpoints[3] = x1;
                _ypoints[3] = y1;
                break;

            case STATE_EAST:
                tempx = (x1 + x2 - 1) / 2;
                tempy = (y1 + y2  + 1) / 2;
                _xpoints[0] = tempx;
                _ypoints[0] = y1;
                _xpoints[1] = x2;
                _ypoints[1] = tempy;
                _xpoints[2] = tempx;
                _ypoints[2] = y2;
                _xpoints[3] = tempx;
                _ypoints[3] = y1;
                break;

            case STATE_SOUTHEAST:
                _xpoints[0] = x2;
                _ypoints[0] = y1;
                _xpoints[1] = x2;
                _ypoints[1] = y2;
                _xpoints[2] = x1;
                _ypoints[2] = y2;
                _xpoints[3] = x2;
                _ypoints[3] = y1;
                break;

            case STATE_SOUTH:
                tempx = (x1 + x2 + 1) / 2;
                tempy = (y1 + y2  - 1) / 2;
                _xpoints[0] = tempx;
                _ypoints[0] = y2;
                _xpoints[1] = x1;
                _ypoints[1] = tempy;
                _xpoints[2] = x2;
                _ypoints[2] = tempy;
                _xpoints[3] = tempx;
                _ypoints[3] = y2;
                break;

            case STATE_SOUTHWEST:
                _xpoints[0] = x1;
                _ypoints[0] = y1;
                _xpoints[1] = x2;
                _ypoints[1] = y2;
                _xpoints[2] = x1;
                _ypoints[2] = y2;
                _xpoints[3] = x1;
                _ypoints[3] = y1;
                break;

            case STATE_WEST:
                tempx = (x1 + x2 + 1) / 2;
                tempy = (y1 + y2  - 1) / 2;
                _xpoints[0] = x1;
                _ypoints[0] = tempy;
                _xpoints[1] = tempx;
                _ypoints[1] = y1;
                _xpoints[2] = tempx;
                _ypoints[2] = y2;
                _xpoints[3] = x1;
                _ypoints[3] = tempy;
                break;

            case STATE_NORTHWEST:
                _xpoints[0] = x1;
                _ypoints[0] = y1;
                _xpoints[1] = x2;
                _ypoints[1] = y1;
                _xpoints[2] = x1;
                _ypoints[2] = y2;
                _xpoints[3] = x1;
                _ypoints[3] = y1;
                break;
        }
    } 
}
