package com.blazinggames.urcl;

// Java imports
import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.net.URL;

// Spelchan imports
import com.blazinggames.urcl.ui.*;
import com.blazinggames.urcl.data.*;

/**
 * Ultimate Retro Common Library
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * The applet class that uses URCL displays.
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class URCLApplet 
extends Applet
implements Runnable, DisplayHandler, MouseMotionListener, MouseListener, KeyListener
{

// *********************
// ***** CONSTANTS *****
// *********************
    public static final int DISPLAY_ERROR = 0;
    public static final int DISPLAY_TITLE = 1;

// *********************
// ***** VARIABLES *****
// *********************

    protected boolean _debug = false;
    protected AudioClip _silentSound = null;
    
    // double buffer
    protected Image _imgDubBuf = null;
    protected Graphics _gDubBuf = null;
    protected LoadingBar _loader = null;
    
    // Resources
    protected ResMan _res;
    protected Config _config;
    
    // Display
    protected boolean _error = false;
    protected Display _currentDisplay;
    protected Display _errorDisplay;
    protected Display _titleDisplay;
    
    // timing thread
    Thread _runner = null;
    protected int _tickMilliSeconds = 100;
    
    int count;
    

// *********************************
// ***** METHODS - From Applet *****
// *********************************

	/**
	 * Applet initialization. 
	 */
    public void init()
    {
        System.out.println("This applet uses the Ultimate Retro Common Library (URCL)");
        System.out.println("URCL Copyright "+((char)169)+" 2001,2003-2004 Blazing Games Inc. All Rights Reserved.");
        log("URCLApplet::init() called!");
        _loader = new LoadingBar(this, getName());
        addMouseMotionListener(this);
        addMouseListener(this);
        addKeyListener(this);
        String s = getParameter("SILENCE");
        if (s != null)
        {
            _silentSound = getAudioClip(getCodeBase(), s);
            _silentSound.loop();
        }
    }

	/** 
	 * Starts the timer
	 */
    public void start()
    {
        log("URCLApplet::start() called!");
        if (_runner == null)
        {
            _runner = new Thread(this);
            _runner.start();
        }
        requestFocus();
    }

	/**
	 * Stops the timer
	 */
    public void stop()
    {
        log("URCLApplet::stop() called!");
        if (_runner != null)
        {
            _runner.stop();
            _runner = null;
        }
    }

	/**
	 * destroys the applet
	 */
    public void destroy()
    {
        log("URCLApplet::destroy() called!");
        if (_runner != null)
        {
            _runner.stop();
            _runner = null;
        }

        if (_silentSound != null)
        {
            _silentSound.stop();
        }
    }


// ***********************************
// ***** METHODS - From Runnable *****
// ***********************************

	/**
	 * timing loop
	 */
    public void run()
    {
        log("URCLApplet::run() called!");
        initDisplays();
        try{Thread.sleep(1000);}catch(Exception ignore){}
        showDisplay(DISPLAY_TITLE);
        while (true)
        {
            try
            {
                Thread.sleep(_tickMilliSeconds);
                tick();
            }
            catch(Exception ignore) {}
        }
    }
    
   /**
    * called every time timing loop has finished. As the next timing interval
    * won't start until this funtion has returned, code placed in overriden
    * versions of this should execute quickly.
    */
    public void tick()
    {
        if (_currentDisplay != null)
            _currentDisplay.tick(100);
    }
   

// **********************************************    
// ***** METHODS - From MouseMotionListener *****
// **********************************************
    
    /**
     * Method from MouseMotionListener.
     * 
     * @param e mouse event that occurred
     */
    public void mouseDragged(MouseEvent e)
    {
        if (_currentDisplay != null)
        {
            _currentDisplay.mouse_move(e.getX(), e.getY(), true);
        }
    }

	/**
	 * Method from MouseMotionListener.
	 * 
	 * @param e mouse event that occurred
	 */
    public void mouseMoved(MouseEvent e)
    {
        if (_currentDisplay != null)
        {
            _currentDisplay.mouse_move(e.getX(), e.getY(), false);
        }
    }


// ****************************************
// ***** METHODS - From MouseListener *****
// ****************************************

   /**
	* Method from MouseListener.
	* 
	* @param e mouse event that occurred
	*/
    public void mouseClicked(MouseEvent e)
    {
        // Ignored but here as is part of interface
    }

	/**
	 * Method from MouseListener.
	 * 
	 * @param e mouse event that occurred
	 */
    public void mousePressed(MouseEvent e)
    {
        if (_currentDisplay != null)
            _currentDisplay.mouse_down(e.getX(), e.getY());
    }

	/**
	 * Method from MouseListener.
	 * 
	 * @param e mouse event that occurred
	 */
    public void mouseReleased(MouseEvent e)
    {
        if (_currentDisplay != null)
            _currentDisplay.mouse_up(e.getX(), e.getY());
    }

	/**
	 * Method from MouseListener.
	 * 
	 * @param e mouse event that occurred
	 */
    public void mouseEntered(MouseEvent e)
    {
        // Ignored but here as is part of interface
    }

	/**
	 * Method from MouseListener.
	 * 
	 * @param e mouse event that occurred
	 */
    public void mouseExited(MouseEvent e)
    {
        // Ignored but here as is part of interface
    }


// **************************************
// ***** METHODS - From KeyListener *****
// **************************************

   /**
	* Method from KeyListener.
	* 
	* @param e Keyboard event that occurred
	*/
    public void keyTyped(KeyEvent e)
    {
        // Ignored but here as is part of interface
    }

	/**
	 * Method from KeyListener.
	 * 
	 * @param e Keyboard event that occurred
	 */
    public void keyPressed(KeyEvent e)
    {
        if (_currentDisplay != null)
        {
            _currentDisplay.key_down(e.getKeyCode(), e.getKeyChar());
        }
    }

	/**
	 * Method from KeyListener.
	 * 
	 * @param e Keyboard event that occurred
	 */
    public void keyReleased(KeyEvent e)
    {
        if (_currentDisplay != null)
        {
            _currentDisplay.key_up(e.getKeyCode(), e.getKeyChar());
        }
    }

// *****************************
// ***** METHODS - Display *****
// *****************************

	/**
	 * Used so that progress bar title can be changed easily.
	 */
    public String getName()
    {
        return "Ultimate Retro Common Library Applet";
    }
    
    /**
     * Creates a double buffer that the game can use.
     */
    protected void createDoubleBuffer()
    {
        Dimension d = getSize();
        _imgDubBuf = createImage(d.width, d.height);
        _gDubBuf = _imgDubBuf.getGraphics();
    }
    
    /**
     * This is overrided so that the screen won't be cleared (no flicker).
     * We don't need to clear the screen anyway as the double buffer covers
     * the entire screen.
     * 
     * @param g graphics context
     */
    public void update(Graphics g)
    {
        paint(g);
    }    
    
    /**
     * Draws the double buffer
     * 
     * @param g graphics context
     */
    public void paint(Graphics g)
    {
        if (_gDubBuf == null)
        {
            createDoubleBuffer();
        }
        if (_currentDisplay == null)
        {
            if (_loader != null)
                _loader.render(g, this);
            return;
        }
        else
            _currentDisplay.render(_gDubBuf, this);
        g.drawImage(_imgDubBuf, 0, 0, this);
    }
    
    /**
     * This is where you initialize any displays that you are using 
     */
    protected void initDisplays()
    {
        _titleDisplay = new Display();
        _titleDisplay.initialize(this);
    }
    
    /**
     * Override if you add new displays so that you can change to them
     */
    public final void showDisplay(int id)
    {
        if (_error)
            id = DISPLAY_ERROR;
        if (_currentDisplay != null)
            _currentDisplay.end();
        Display temp = getDisplay(id);
        if (temp == null)
        {
            showError("Unknown display mode requested!");
            return;
        }
        _currentDisplay = temp;
        _currentDisplay.begin();
        repaint();
    }
    
    /**
     * returns the current display.
     * @return current display
     */
    protected Display getDisplay(int id)
    {
        Display ret = null;
        switch (id)
        {
            case DISPLAY_ERROR :
                ret = _errorDisplay;
                break;
                
            case DISPLAY_TITLE :
                ret = _titleDisplay;
                break;
                
            default :
                ret = null;
                break;
        }
        return ret;
    }
    
    /**
     * sets up the error screen to show the passed error message
     * 
     * @param s error message to show
     */
    public void showError(String s)
    {
        _error = true;
        String m[] = new String[3];
        m[0] = "AN ERROR HAS OCCURRED!";
        m[1] = s;
        m[2] = "Please close this browser window";
        
        if (_errorDisplay == null)
        {
            _errorDisplay = new Display();
            _errorDisplay.message = m;
        }
        showDisplay(DISPLAY_ERROR);
    }
    
// ******************************************
// ***** METHODS - from DisplayListener *****
// ******************************************

	/**
	 * returns the resource manager.
	 * 
	 * @return resource manager.
	 */
    public ResMan getResMan()
    {
        if (_res == null)
        {
            _res = new ResMan(this);
        }
        return _res;
    }
    
    /**
     * returns the configuration file.
     * 
     * @return config
     */
    public Config getConfig()
    {
log("URCLApplet::getConfig()");
        if (_config == null)
        {
            _config = new Config();
            try
            {
                URL url = new URL(getCodeBase(), getParameter("CONFIG"));
                _config.loadConfigURL(url);
                _debug = (_config.getMode() == Config.TEST_CONFIG);
            }
            catch (Exception e)
            {
                log(e, "getConfig failed: " + e.getMessage());
                showError("Problem with configuration file");
            }
            return _config;
        }
        
        return _config;
    }
    
    /**
     * returns the container the applet is in
     * 
     * @return container
     */
    public Container getContainer()
    {
        return this;
    }

	/**
	 * Used to tell the applet that the display needs to be refreshed
	 * @param caller display requesting to be refreshed
	 */
    public void refreshDisplay(Display caller)
    {
        repaint();
    }
    
    /**
     * Used for handling any special display events. By default, the only event
     * that is known is the LOADING command that adjusts the loading bar. this
     * is the ideal place to add commands for switching displays if your game
     * has more than one display.
     * @param caller display that is sending command
     * @param command the command to execute
     */
    public void displayEvent(Display caller, String command)
    {
        log(caller.getClass().getName() + " called for command:" + command);
        if (command.startsWith("LOADING"))
        {
            String s = command.substring(command.lastIndexOf(" ")+1);
            int n = Integer.parseInt(s);
            _loader.setLoadPercent(n);
            repaint();
        }
    }

	/**
	 * When debug mode is enabled, sends message to the console window 
	 * otherwise does nothing.
	 * 
	 * @param s message string to display 
	 */
    public void log(String s)
    {
        if (_debug)
            System.out.println(s);
    }
    
	/**
	 * When debug mode is enabled, sends message to the console window 
	 * otherwise does nothing.
	 * 
	 * @param s message string to display 
	 * @param t exception to dump
	 */
    public void log(Throwable t, String s)
    {
        if (_debug)
        {
            System.out.println(s);
            t.printStackTrace();
        }
    }
    
	/**
	 * Debbuging function that throws and logs an exception if the passed 
	 * condition is false
	 * 
	 * @param b a boolean expression that should resolve to true.
	 * @param s message string describing reason for assertion 
	 */
    public void assert(boolean b, String s)
    {
        if (!b)
        {
            Throwable t = new Throwable();
            log(t, "ASSERTION FAILURE: " + s);
        }
    }
    
}
