package com.blazinggames.urcl.ui.comp;

// Java imports
import java.awt.*;

// Spelchan imports
import com.blazinggames.urcl.ui.sprite.*;

/**
 * Ultimate Retro Common Library
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * Your typical checkbox component.
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class SCheckbox
extends Comp
{
// *********************
// ***** CONSTANTS *****
// *********************

    private static final int GAP_SIZE = 5;


// *********************
// ***** VARIABLES *****
// *********************

    protected int _state;
    protected Color _foreground;
    protected Color _background;

// ************************
// ***** CONSTRUCTORS *****
// ************************

    public SCheckbox(Component c, String s, int x, int y)
    {
        super();
        _sprite = new TextSprite(c, s, x, y);
        _sprite.setSolid(false);
        setBackground(Color.white);
        setForeground(Color.black);
        setBounds(x,y, _sprite.getHeight() + GAP_SIZE + _sprite.getWidth(), _sprite.getHeight());
        _state = 0;
    }


// *******************************
// ***** METHODS - From Comp *****
// *******************************

    /** 
        * Called when mouse button has been released.
        *
        * @param x x coordinate of where mouse was released
        * @param y y coordinate of where mouse was released
        * @return true if component clicked (pressed and released in component)
        */
    public boolean mouse_up(int x, int y)
    {
        boolean b = super.mouse_up(x, y);

        if ( (b) && (_state >= 0) )
        {
            setState(_state + 1);
        }

        return b;
    }

    /**
        * Sets if a component is active or not. A non-active component generally
        * looks different and doesn't respond to events.
        *
        * @param b true if component is to be active, false otherwise
        */
    public void setActive(boolean b)
    {
        super.setActive(b);

        if ( (b) && (_state < 0) )
            _state = 0;
        else if (b == false)
            _state = -1;
    }

    /**
        * Draws the component.
        *
        * @param g Graphics Context to draw into
        * @param c Component class that some awt commands need
        */
    public void render(Graphics g, Component c)
    {
        _changed = false;
        int h = _sprite.getHeight();
        g.setColor(_background);
        g.fillRect(_bounds.x, _bounds.y, h, h);
        g.setColor(_foreground);
        g.drawRect(_bounds.x, _bounds.y, h, h);
        g.drawRect(_bounds.x+1, _bounds.y+1, h-2, h-2);
        if (_state == 1) {
            g.drawLine(_bounds.x+1, _bounds.y, _bounds.x + h, _bounds.y + h - 1);
            g.drawLine(_bounds.x, _bounds.y, _bounds.x + h, _bounds.y + h);	    
            g.drawLine(_bounds.x, _bounds.y + 1, _bounds.x + h - 1, _bounds.y + h);
            g.drawLine(_bounds.x + h, _bounds.y, _bounds.x, _bounds.y + h - 1);
            g.drawLine(_bounds.x + h + 1, _bounds.y, _bounds.x, _bounds.y + h);
            g.drawLine(_bounds.x + h, _bounds.y + 1, _bounds.x - 1, _bounds.y + h);
        }

        _sprite.move(_bounds.x + GAP_SIZE + _sprite.getHeight(), _bounds.y);
        _sprite.setVisible(_isVisible);
        _sprite.setBounding(_isSelected);
        _sprite.render(g, c);
        _sprite.clean();
    }


    // *****************************************************
    // ***** METHODS - Information getting and setting *****
    // *****************************************************

    /** 
        * Changes the state of the toggle to the indicated state.
        *
        * @param n new state for the component
        */
    public void setState(int n)
    {
        if (_isActive == false)
            return;
        if (n >= 2)
        {
            _state = 0;
        }
        else
        {
            _state = n;
        }
    }

    /**
        * Goes to the next state in the toggle sequence
        */
    public void nextState()
    {
        setState(_state+1);
    }

    /**
        * @return Current state of the toggle (-1 if disabled)
        */
    public int getState()
    {
        return _state;
    }

    public void setForeground(Color c)
    {
        _foreground = c;
        ((TextSprite)_sprite).setFontColor(c);
    }

    public void setBackground(Color c)
    {
        _background = c;
    }

    public void setText(Component c, String s)
    {
        ((TextSprite)_sprite).setText(c, s);
    }

    public void setFont(Component c, Font f)
    {
        ((TextSprite)_sprite).setFont(c, f);
    }

    public void setFormat(int n)
    {
        ((TextSprite)_sprite).setFormat(n);
    }
    
}
